C Sample use of micromagnetic routines from F77.
C Compute a classical Landau-Lifshitz wall profile.
C
C Copyright (C) 2007  Seagate Technology
C
C This program is free software; you can redistribute it and/or modify
C it under the terms of the GNU General Public License as published by
C the Free Software Foundation; either version 2 of the License, or
C (at your option) any later version.
C
C This program is distributed in the hope that it will be useful,
C but WITHOUT ANY WARRANTY; without even the implied warranty of
C MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C GNU General Public License for more details.
C
C You should have received a copy of the GNU General Public License along
C with this program; if not, write to the Free Software Foundation, Inc.,
C 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
C
C The author can be reached by electronic mail at Pierre.Asselin@seagate.com


C Trivial preconditioning routine
C -- FIXME --  provide a default in the library
      subroutine llw_precond(ns, s, h, delta_s)
      implicit none
      integer ns
      double precision s(3,ns), h(3,ns), delta_s(3,ns)
      integer i, k
      do i= 1, ns
        do k= 1, 3
          delta_s(k,i)= h(k,i)
        end do
      end do
      return
      end

C Field computation subroutine
      subroutine llw_field(ns, s, h)
      implicit none
      integer ns                ! input, number of spins
      double precision s(3,ns)  ! input
      double precision h(3,ns)  ! output

C     In a realistic example, this common block would be in an include file.
      double precision A, Ku, Ko, ea(3), ma(3), sleft(3), sright(3)
      common/params/ A, Ku, Ko, ea, ma, sleft, sright
      save /params/

      integer i, k
      double precision tmp

      do i= 1, ns
        do k= 1, 3
          h(k,i)= 0
        end do
      end do

C     Exchange.  (Negative derivative of exchange energy.)
      do i= 1, ns-1
        do k= 1, 3
          tmp= A*(s(k,i+1)-s(k,i))
          h(k,i+1)= h(k,i+1) - tmp
          h(k,i)  = h(k,i)   + tmp
        end do
      end do

C     Exchange, endpoints
      do k= 1, 3
        tmp= A*(s(k,1)-sleft(k))
        h(k,1) = h(k,1)  - tmp
        tmp= A*(sright(k)-s(k,ns))
        h(k,ns)= h(k,ns) + tmp
      end do

C     Anisotropy
      do i= 1, ns
        tmp= Ku*(s(1,i)*ea(1) + s(2,i)*ea(2) + s(3,i)*ea(3))
        do k= 1, 3
          h(k,i)= h(k,i) + tmp*ea(k)
        end do
        tmp= Ko*(s(1,i)*ma(1) + s(2,i)*ma(2) + s(3,i)*ma(3))
        do k= 1, 3
          h(k,i)= h(k,i) + tmp*ma(k)
        end do
      end do

      return
      end



! /* this stolen from the rand(3) manpage */
      double precision function badrandom(dummy)
      integer dummy
      integer badstate
      data badstate/1/
      save badstate
      badstate= badstate * 1103515245 + 12345;
      badrandom= mod(badstate/65536, 32768)/32768.0
      end



      program main
      implicit none
      integer   nspins
      parameter(nspins=100)

      double precision A, Ku, Ko, ea(3), ma(3), sleft(3), sright(3)
      common/params/ A, Ku, Ko, ea, ma, sleft, sright
      save /params/

      integer i, k, istatus
      double precision s0(3, nspins), s(3, nspins)
      double precision w(nspins)
      double precision resid2
      double precision ell, ell2, rlambda
      double precision tmp, tmp1,tmp3

      external llw_field, llw_precond
      integer  dmmu, dmmup, dmmupf
      external dmmu, dmmup, dmmupf
      double precision badrandom
      external         badrandom

      integer   SPINS,    GNUPLOT
      parameter(SPINS=10, GNUPLOT=11)

C     Endpoints
      sleft(1)= 0
      sleft(2)= 0
      sleft(3)= -1
      sright(1)= 0
      sright(2)= 0
      sright(3)= 1

C     Exchange, anisotropy.
C     Note: A is in "ergs" * (interval length),
C     Ku is in "ergs" per (interval length).
C     Choose values so sqrt(A/Ku) is a dozen intervals or so.
C     Ko is the medium axis anisotropy ("orthorhombic" anisotropy).
C     I choose 0.1*Ku just to break the degeneracy in phi.
C     In a real problem with magnetostatics the minimum would not be
C     degenerate but here we skip magnetostatics entirely.
      A= 144d0
      Ku= 1d0
      Ko= 0.1d0
C
C     "ea" is the easy axis, "ma" is the medium axis.
      ea(1)= 0
      ea(2)= 0
      ea(3)= 1
      ma(1)= 1
      ma(2)= 0
      ma(3)= 0

C     Initial state
      do i= 1, nspins
        tmp1= (4d0*i*(+nspins-i))/nspins**2
        tmp3= (2d0*i-nspins)/nspins
        tmp= 1.0/sqrt(tmp1**2 + tmp3**2)
        s0(1,i)= tmp*tmp1
        s0(2,i)= 0.2*s0(1,i)
        s0(3,i)= tmp*tmp3
      end do

      open(unit=SPINS, file='llw-dat.out')
      open(unit=GNUPLOT, file='llw-gp.out')

C     Crunch time.
C     Find equilibrium micromagnetic state.
      print *, 'Micromagnetic minimum'
      istatus= dmmu(
     &  nspins, s0, 1d-10, resid2, 1000, llw_field, llw_precond
     &)
 
      print *, 'dmmu returns', istatus
      print *, 'final residual', sqrt(resid2)

      write(SPINS,'(A)') '# n    mx    my    mz'
      do i= 1, nspins
        write(SPINS,'(I4,3F10.5)') i, (s0(k,i), k= 1, 3)
      end do

      write(GNUPLOT,'(A)') '# [Linux] pass this file to gnuplot'
      write(GNUPLOT,'(A)') '# to plot the LL wall profile generated'
      write(GNUPLOT,'(A)') '# by example.f'
      write(GNUPLOT,'()')
      write(GNUPLOT,'(A)') 'set data style linespoints'
      write(GNUPLOT,'(A)') 'set key left'
      write(GNUPLOT,'(A)') 'set title ''Landau-Lifshitz wall profile'''
      write(GNUPLOT,'(A)') 'plot ''llw-dat.out'' index 0 using 1:4'
      write(GNUPLOT,'(A)') 'replot ''llw-dat.out'' index 0 using 1:3'
      write(GNUPLOT,'(A)') 'replot ''llw-dat.out'' index 0 using 1:2'
      write(GNUPLOT,'(A)') 'pause -1'

C     New state randomly perturbed from the reference state.
C     Doesn't have to be normalized to unit length yet.
      print *
      print *, 'Perturbing the minimum...'
      do i= 1, nspins
        do k= 1, 3
          s(k,i)= s0(k,i) + 0.01d0*(badrandom(0)-0.5d0)
        end do
      end do

C     Jiggle the new state to bring it to average Euclidean distance 0.01
C     from the reference.  Takes care of normalizing the spins too.
C     Weigh all spins equally in the averaging.
      print *, 'Forcing Euclidean distance from reference...'
      do i= 1, nspins
        w(i)= 1         ! no need to normalize them to unit sum
      end do
      istatus= dmmupf(nspins, s, s0, w, 1d-4, ell2)
      print *, 'dmmupf returns', istatus
      ell= sqrt(ell2)
      print *, 'RMS Euclidean distance=', ell

C     Minimize the energy subject to staying at that Euclidean distance.
      print *, 'Constrained minimization'
      istatus= dmmup(
     &  nspins, s, 1d-10, resid2, 1000, llw_field, llw_precond,
     &  s0, w, ell2, rlambda
     &)
      print *, 'dmmup returns', istatus
      print *, 'final residual', sqrt(resid2)
      print *, 'Lagrange multiplier', rlambda

      write(SPINS, '()')
      write(SPINS, '()')
      do i= 1, nspins
        write(SPINS,'(I4,3F10.5)') i, ((s(k,i)-s0(k,i))/ell, k= 1, 3)
      end do

      write(GNUPLOT,'()')
      write(GNUPLOT,'(A)') 'set title ''Lowest eigenvector'''
      write(GNUPLOT,'(A)') 'plot ''llw-dat.out'' index 1 using 1:4'
      write(GNUPLOT,'(A)') 'replot ''llw-dat.out'' index 1 using 1:3'
      write(GNUPLOT,'(A)') 'replot ''llw-dat.out'' index 1 using 1:2'
      write(GNUPLOT,'(A)') 'pause -1'


C     Now force the distance higher to get a visible difference
C     from the reference state.
      print *
      print *, 'Increasing distance from reference'
      ell= 0.5
      istatus= dmmupf(nspins, s, s0, w, ell*ell, ell2)
      print *, 'dmmupf returns', istatus
      ell= sqrt(ell2)
      print *, 'RMS Euclidean distance now=', ell

      istatus= dmmup(
     &  nspins, s, 1d-10, resid2, 1000, llw_field, llw_precond,
     &  s0, w, ell2, rlambda
     &)
      print *, 'dmmup returns', istatus
      print *, 'final residual', sqrt(resid2)
      print *, 'Lagrange multiplier', rlambda

      write(SPINS, '()')
      write(SPINS, '()')
      do i= 1, nspins
        write(SPINS,'(I4,3F10.5)') i, (s(k,i), k= 1, 3)
      end do

      write(GNUPLOT,'()')
      write(GNUPLOT,'(A)') 'set title ''constrained wall profile'''
      write(GNUPLOT,'(A)') 'plot ''llw-dat.out'' index 0 using 1:4'
      write(GNUPLOT,'(A)') 'replot ''llw-dat.out'' index 0 using 1:3'
      write(GNUPLOT,'(A)') 'replot ''llw-dat.out'' index 0 using 1:2'
      write(GNUPLOT,'(A)') 'replot ''llw-dat.out'' index 2 using 1:4'
      write(GNUPLOT,'(A)') 'replot ''llw-dat.out'' index 2 using 1:3'
      write(GNUPLOT,'(A)') 'replot ''llw-dat.out'' index 2 using 1:2'
      write(GNUPLOT,'(A)') 'pause -1'


      close(unit=GNUPLOT)
      close(unit=SPINS)

      end
