/* $Id: mmu.c,v 1.8.2.1 2007/04/08 23:32:01 pa Exp $
 * Fortran interface to minimu.c
 *
 * Copyright (C) 2007  Seagate Technology
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * The author can be reached by electronic mail at Pierre.Asselin@seagate.com
 */

#include "config.h"

#ifdef HAVE_WINDOWS_H
#include <windows.h>
#endif

#include <minimu.h>

/* Fortran field and preconditioning callbacks.
 * Essentially, like the C versions but without the context pointer
 * and writable in Fortran.
 *     subroutine field(n, s, h)
 *     subroutine precond(n, s, h, delta_s)
 *     integer n; real s(*), h(*), delta_s(*)
 */
typedef F77_SUB_RETURN_T (F77_CALL *f77_field_sub)(
	F77_INT_T *n,
	const FLOAt s[],
	FLOAt h[]
);

typedef F77_SUB_RETURN_T (F77_CALL *f77_precond_sub)(
	F77_INT_T *n,
	const FLOAt s[],
	const FLOAt h[],
	FLOAt delta_s[]
);

/* Glue between minimu() and the Fortran callbacks */
struct fortran_wrap{
	f77_field_sub	field;
	f77_precond_sub	precond;
};
static void field_wrap(
	int n, const FLOAt s[], FLOAt h[], void *context
) {
	struct fortran_wrap *fw= context;
	F77_INT_T fn= n;
	fw->field(&fn, s, h);
}
static void precond_wrap(
	int n, const FLOAt s[], const FLOAt h[], FLOAt delta_s[],
	void *context
) {
	struct fortran_wrap *fw= context;
	F77_INT_T fn= n;
	fw->precond(&fn, s, h, delta_s);
}


/* integer function smmu(  &
 *     n, s, rtorq2, rtorq2_actual, itermax, field_sub,  precond_sub   &
 *  )
 */
#ifdef USE_FLOAT
F77_INT_T F77_CALL F77_FUNC(smmu,SMMU)(
#else
F77_INT_T F77_CALL F77_FUNC(dmmu,DMMU)(
#endif /* USE_FLOAT */
	const F77_INT_T *n,
	FLOAt s[],
	const FLOAt *rtorq2,
	FLOAt *rtorq2_actual,
	const F77_INT_T *max_iterations,
	f77_field_sub user_field_sub,
	f77_precond_sub user_precond_sub
)
{
	struct fortran_wrap fw= {
		user_field_sub,		/* .field */
		user_precond_sub,	/* .precond */
	};
	return DECORATE(minimu)(
		*n, s, 0,		/* system size, state, no scratch */
		*rtorq2, rtorq2_actual,	/* tolerance */
		*max_iterations,	/* iterations not to exceed */
		field_wrap, precond_wrap, &fw,	/* wrapped Fortran */
		0, 0			/* no logging */
	);
}
