/* $Id: mmup.c,v 1.4.2.1 2007/04/08 23:32:01 pa Exp $
 * Fortran interface to minimu_push.c
 *
 * Copyright (C) 2007  Seagate Technology
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * The author can be reached by electronic mail at Pierre.Asselin@seagate.com
 */

#include "config.h"

#ifdef HAVE_WINDOWS_H
#include <windows.h>
#endif

#include <minimu_push.h>

/* Fortran field and preconditioning callbacks.
 * -- FIXME -- duplicate code with mmu.c
 *     subroutine field(n, s, h)
 *     subroutine precond(n, s, h, delta_s)
 *     integer n; real s(*), h(*), delta_s(*)
 */
typedef F77_SUB_RETURN_T (F77_CALL *f77_field_sub)(
	F77_INT_T *n,
	const FLOAt s[],
	FLOAt h[]
);

typedef F77_SUB_RETURN_T (F77_CALL *f77_precond_sub)(
	F77_INT_T *n,
	const FLOAt s[],
	const FLOAt h[],
	FLOAt delta_s[]
);

/* Glue between minimu_push() and the Fortran callbacks */
struct fortran_wrap{
	f77_field_sub	field;
	f77_precond_sub	precond;
};
static void field_wrap(
	int n, const FLOAt s[], FLOAt h[], void *context
)
{
	struct fortran_wrap *fw= context;
	F77_INT_T fn= n;
	fw->field(&fn, s, h);
}
static void precond_wrap(
	int n, const FLOAt s[], const FLOAt h[], FLOAt delta_s[],
	void *context
) {
	struct fortran_wrap *fw= context;
	F77_INT_T fn= n;
	fw->precond(&fn, s, h, delta_s);
}


/* Constrained micromagnetic minimization
 *
 * integer function smmup(  &
 *     n, &		! (in) number of spins
 *     s, &		! (inout) s(3*n), initial state, replaced by minimum
 * ! convergence control
 *     rtorq2, &	! (in) exit threshold on residuals
 *     rtorq2_actual, &	! (out) residuals actually achieved
 *     itermax, &	! (in) iteration count not to exceed
 * ! physics
 *     field_sub, &	! (in) subroutine to compute fields
 *     precond_sub &	! (in) preconditioning subroutine
 * ! supplementary constraint
 *     s0, &		! (in) s0(3*n), reference state
 *     w, &		! (in) w(n), weights
 *     rl2, &		! (in) squared Euclidian distance away from s0()
 *     rlambda, &	! (out) Lagrange multiplier
 *  )
 */
#ifdef USE_FLOAT
F77_INT_T F77_CALL F77_FUNC(smmup,SMMUP)(
#else
F77_INT_T F77_CALL F77_FUNC(dmmup,DMMUP)(
#endif /* USE_FLOAT */
	const F77_INT_T *n,
	FLOAt s[],
	const FLOAt *rtorq2,
	FLOAt *rtorq2_actual,
	const F77_INT_T *max_iterations,
	f77_field_sub user_field_sub,
	f77_precond_sub user_precond_sub,
	const FLOAt *s0,
	const FLOAt *w,
	const FLOAt *rl2,
	FLOAt *rlambda
)
{
	struct fortran_wrap fw= {
		user_field_sub,		/* .field */
		user_precond_sub,	/* .precond */
	};
	return DECORATE(minimu_push)(
		*n, s, 0,		/* system size, state, no scratch */
		*rtorq2, rtorq2_actual,	/* tolerance */
		*max_iterations,	/* iterations not to exceed */
		field_wrap, precond_wrap,	/* wrapped Fortran */
		s0, w, *rl2, rlambda,	/* supplementary constraint */
		&fw,			/* Fortran wrap context */
		0, 0			/* no logging */
	);
}


/* Normalization, force a micromagnetic state to a given Euclidean
 * distance from a reference state.
 * Returned value is 0 for ok, negative if error.
 * -- FIXME -- Fortran style would be a subroutine with error returns
 *  through a dummy argument.
 *
 * integer function smmupf(
 *      n, &	! number of spins
 *      s, &	! s(3*n), initial state, replaced
 *      s0, &	! s0(3*n), reference state
 *      w, &	! w(n), weights
 *      rl2,	! requested squared Euclidean distance
 *      rl2a, &	! squared distance actually achieved
 * )
 */
#ifdef USE_FLOAT
F77_INT_T F77_CALL F77_FUNC(smmupf,SMMUPF)(
#else
F77_INT_T F77_CALL F77_FUNC(dmmupf,DMMUPF)(
#endif /* USE_FLOAT */
	const F77_INT_T *n,
	FLOAt s[],
	const FLOAt s0[],
	const FLOAt w[],
	const FLOAt *rl2,
	FLOAt *rl2_actual
)
{
	return DECORATE(minimu_push_make_feasible)(
		*n, s, s0, w, *rl2, rl2_actual
	);
}
