/* cgeq_simplehook.c -- handy logging hook for cgeq() and friends.
 *
 * Copyright (C) 2007  Seagate Technology
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * The author can be reached by electronic mail at Pierre.Asselin@seagate.com
 */
#include "cgeq.h"
#include <stdio.h>
#include <stdarg.h>

/* keep in sync with cgeq.h */
static struct lscodes {
	unsigned int bit;
	const char *name;
} codes[]= {
	{CGEQ_LS_CLAMP,      "clamped"},
	{CGEQ_LS_SADDLEPT,   "saddle"},
	{CGEQ_LS_BRACKETMAX, "no_bracket"},
	{CGEQ_LS_BISECTMAX,  "bisect_fail"},
	{CGEQ_LS_NONLINEAR,  "nonlinear"},
	{CGEQ_LS_CONJUG,     "conjugacy"},
	{CGEQ_LS_ERROR,      "error"},
	{CGEQ_LS_FATAL,      "fatal"},
};
static const int ncodes= sizeof(codes)/sizeof(codes[0]);

int cgeq_simple_log_hook(
	int opcode,
	void *logging_context,
	...
) {
	FILE *out= logging_context;
	va_list ap;
	int iter; double r2; unsigned int ls_status;
	int i;

	if(0 == out) return 0;
	va_start(ap, logging_context);
	switch(opcode) {
	case CGEQ_LOG_NEWPOINT:
		iter= va_arg(ap, int);
		r2= va_arg(ap, double);
		ls_status= va_arg(ap, unsigned int);
		fprintf(out, "% 4d  % 13.5e  %#4x\n",
			iter, r2, ls_status
		);
	break;
	case CGEQ_LOG_RESTART:
		ls_status= va_arg(ap, unsigned int);
		fprintf(out, "# restart %#4x", ls_status);
		if(ls_status) {
			fputs(" (", out);
			for(i= 0; i<ncodes; i++) {
				if(ls_status & codes[i].bit) {
					fputc(' ', out);
					fputs(codes[i].name, out);
				}
			}
		}
		fputs(" )\n\n", out);
	break;
	}
	va_end(ap);
	return 0;
} /* cgeq_simple_log_hook() */

/* This hook is called by most of the unit tests and doesn't need
 * a test of its own.
 */
