#ifndef MINIMU_P_H
#define MINIMU_P_H

/* $Id: minimu_P.h,v 1.8.2.1 2007/04/08 23:32:01 pa Exp $
 * Private header for micromagnetics.
 * Must include "config.h" first.
 * Also #define USE_FLOAT before including "config.h".
 *
 * Copyright (C) 2007  Seagate Technology
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * The author can be reached by electronic mail at Pierre.Asselin@seagate.com
 *
 * The functions and data structures in this file used to be private to
 * minimu.c, but I needed to reuse them from minimu_push.c .
 * Since this is a private header it is allowed to be ugly.
 */
#include "minimu.h"

/* Glue between cgeq() and the user callbacks.
 * We need this glue layer because the argument "n" means "number
 * of spins" to the users and "number of variables" (3 times
 * larger) for cgeq().
 */

/* struct to hold the user's callbacks and context */
struct DECORATE(minimu_wrap) {
	DECORATE(minimu_field_proc) get_field;
	DECORATE(minimu_precond)    get_steepest;
	void *minimu_context;
};

/* callbacks that we will pass to cgeq() */
extern void DECORATE(minimu_grad_wrap)(
	int n, const FLOAt state[], FLOAt gradient[], void *context
);
extern void DECORATE(minimu_precond_wrap)(
	int n, const FLOAt state[], const FLOAt grad[],
	FLOAt step[], void *context
);


/* routines to handle regular micromagnetic constraints */
extern int DECORATE(minimu_normalize)(int n, FLOAt s[], void *ignored);
extern void DECORATE(minimu_project)(
	int n, const FLOAt s[], FLOAt ds[], void *ignored
);
extern int DECORATE(minimu_stepsize)(
	int n, const FLOAt s[], const FLOAt ds[],
	FLOAt *s0, FLOAt *smax, void *ignored
);


#endif /* MINIMU_P_H */
